﻿using System;
using System.Drawing;

using MonoTouch.Foundation;
using MonoTouch.UIKit;

using CGGlyph = System.UInt16;
using MonoTouch.ObjCRuntime;
using MonoTouch.CoreGraphics;

namespace NucliOSHomeAnimation_CS
{
	public partial class NucliOSHomeAnimation_CSViewController : UIViewController
	{
		UIImageView _backgroundView;
		SizeF _viewSize;

		float _originVal;
		float _buttonSize;
		UIView _containerView;
		NSArray _bubbles, _circleSizes, _launchLabelNames;

		UIImageView _circle1, _circle2, _circle3, _circle4, _circle5, _circle3_2, _circle4_2, _circleOverLay, _labelUnderline, _logoIV, _igLogo;
		UILabel _launchLabel, _iosLabel;

		private bool IsPhone 
		{
			get { return UIDevice.CurrentDevice.UserInterfaceIdiom == UIUserInterfaceIdiom.Phone; }
		}

		private Bubble CreateBubble(NSArray numbers, NSArray features, RectangleF frame, bool isRed, float fontSize, NSString key)
		{
			Bubble bubble = new Bubble (isRed, new NSString (numbers.ValueAt (0)), fontSize);
			bubble.Frame = frame;
			bubble.AddTarget (this, new Selector ("UpdateBubble:"), UIControlEvent.ValueChanged);
			_containerView.Add(bubble);

			UILabel label = SetupLabel (key, bubble);

			bubble.Features = features;
			bubble.Numbers = numbers;

			UIColor color = (isRed) ? UIColor.FromRGBA (241.0f / 255.0f, 108.0f / 255.0f, 39.0f / 255.0f, 1.0f) : UIColor.FromRGBA (13.0f / 255.0f, 87.0f / 255.0f, 134.0f / 255.0f, 1.0f);

			bubble.FeatureLabel = SetupFeatureLabel (color, new NSString (features.ValueAt (0)), label);
			bubble.ControlNameLabel = label;

			return bubble;
		}

		private void Fade(Bubble bubble)
		{
			UIView.Animate (0.75, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
				bubble.Fill.Alpha = 0.7f;
				bubble.ContentLabel.Alpha = 0.001f;
				bubble.FeatureLabel.Alpha = 0.0f;
			}, () => {
				UIView.Animate(0.01, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
					bubble.ContentLabel.Alpha = 0.0f;
				}, () => {
					bubble.FeatureLabel.Text = new NSString(bubble.Features.ValueAt((uint)bubble.Index));
					bubble.FeatureLabel.SizeToFit();
					bubble.ContentLabel.Text = new NSString(bubble.Numbers.ValueAt((uint)bubble.Index));
					bubble.Rotate();

					UIView.Animate(0.75, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
						bubble.Fill.Alpha = 0.0f;
						bubble.ContentLabel.Alpha = 1.0f;
						bubble.FeatureLabel.Alpha = 1.0f;
					}, () => {});
				});
			});
		}

		[Export("UpdateBubble:")]
		private void UpdateBubble(Bubble bubble)
		{
			bubble.Index++;

			if( bubble.Index >= bubble.Features.Count)
				bubble.Index = 0;

			Fade (bubble);
		}

		private float GetRadiansForAngle(float angle)
		{
			return angle * (float)Math.PI / 180;
		}

		private UIImageView SetupCircleImage(int index)
		{
			UIImageView iv = new UIImageView (UIImage.FromBundle (String.Format ("Images/Nuclios-SPLASH_circle{0}.png", index + 1)));
			iv.ContentMode = UIViewContentMode.ScaleAspectFit;
			float size = new NSNumber (_circleSizes.ValueAt ((uint)index)).FloatValue;
			iv.Frame = new RectangleF (this.View.Frame.Size.Width / 2 - size / 2, this.View.Frame.Size.Height / 2 - size / 2, size, size);
			iv.AutoresizingMask = UIViewAutoresizing.FlexibleTopMargin | UIViewAutoresizing.FlexibleLeftMargin | UIViewAutoresizing.FlexibleRightMargin | UIViewAutoresizing.FlexibleBottomMargin;
			this.View.Add (iv);
			return iv;
		}

		private UIImageView SetupImage(NSString imgName, RectangleF frame)
		{
			UIImageView iv = new UIImageView (UIImage.FromBundle (imgName));
			iv.ContentMode = UIViewContentMode.ScaleAspectFit;
			iv.Frame = frame;
			_containerView.AddSubview (iv);
			return iv;
		}

		private UIImageView SetupImageCode(NSString imgName, RectangleF frame)
		{
			UIImageView iv = SetupImage (imgName, frame);
			iv.Alpha = 0.7f;
			return iv;
		}

		private void SetupiOSFrame(float offset)
		{
			float bbase = Math.Min (this.View.Frame.Size.Width, this.View.Frame.Size.Height);
			float largePadding = bbase * 0.02f;
			float smallPadding = bbase * 0.007f;
			float naLabelWidth = bbase * 0.295f;
			float naLabelHeight = naLabelWidth * 0.34f;
			float logoOffset = bbase * 0.025f;
			float labeloffset = bbase * 0.15f;


			_logoIV.Frame = new RectangleF (logoOffset, (this.View.Frame.Size.Height - (naLabelHeight + largePadding)), naLabelWidth, naLabelHeight);
			_iosLabel.Frame = new RectangleF (labeloffset, (_logoIV.Frame.Y + (_logoIV.Frame.Size.Height / 2)) - smallPadding, _iosLabel.Frame.Size.Width, _iosLabel.Frame.Size.Height);

			_logoIV.AutoresizingMask  = UIViewAutoresizing.FlexibleTopMargin | UIViewAutoresizing.FlexibleRightMargin;
			_iosLabel.AutoresizingMask  = UIViewAutoresizing.FlexibleTopMargin | UIViewAutoresizing.FlexibleRightMargin;

		}

		private UILabel SetupLabel(NSString text, Bubble bubble)
		{
			NSString fontstr = IsPhone ? new NSString("30") : new NSString("60");
			float fontSize = float.Parse(fontstr);

			UILabel label = new UILabel ();
			label.Text = text;
			label.Font = UIFont.FromName ("HelveticaNeue", fontSize);
			label.BackgroundColor = UIColor.Clear;
			label.TextColor = UIColor.Black;
			label.TextAlignment = UITextAlignment.Center;
			label.Frame = new RectangleF (bubble.Frame.X + bubble.Frame.Size.Width / 2, bubble.Frame.Size.Height + bubble.Frame.Y, 0, 0);
			label.SizeToFit ();
			_containerView.Add (label);

			return label;
		}

		private UILabel SetupFeatureLabel(UIColor color, NSString text, UILabel otherLabel)
		{
			float bbase = Math.Min(this.View.Frame.Size.Width, this.View.Frame.Size.Height);
			float fontSize = bbase * 0.028f;

			UILabel label = new UILabel ();
			label.BackgroundColor = UIColor.Clear;
			label.TextColor = color;
			label.Text = text;
			label.LineBreakMode = UILineBreakMode.WordWrap;
			label.Font = UIFont.FromName ("HelveticaNeue", fontSize);
			label.Frame = new RectangleF(otherLabel.Frame.X + otherLabel.Frame.Size.Width, otherLabel.Frame.Y, 0, 0);
			label.SizeToFit();
			_containerView.Add (label);
			return label;
		}

		public NucliOSHomeAnimation_CSViewController (IntPtr handle) : base (handle)
		{
		}

		public override void DidReceiveMemoryWarning ()
		{
			base.DidReceiveMemoryWarning ();
		}

		public override bool PrefersStatusBarHidden ()
		{
			return true;
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();

			_backgroundView = new UIImageView (UIImage.FromBundle ("Images/Home_Main_Background.png"));
			_backgroundView.Frame = new RectangleF (0, 0, this.View.Frame.Size.Width, this.View.Frame.Size.Height);
			_backgroundView.AutoresizingMask = UIViewAutoresizing.FlexibleWidth | UIViewAutoresizing.FlexibleHeight;
			_backgroundView.ContentMode = UIViewContentMode.ScaleToFill;
			this.View.Add (_backgroundView);

			float containerSize = Math.Min (this.View.Frame.Size.Width, this.View.Frame.Size.Height) * 0.85f;

			float badgeSize = containerSize * 0.08f;

			float padding = containerSize * 0.017f;

			float chartBubbleSize = containerSize * 0.375f;
			float chartBubbleFontSize = containerSize * 0.2f;
			float chartCodeWidth = containerSize * 0.113f;
			float chartCodeHeight = containerSize * 0.273f;

			float largeOffset = containerSize * 0.025f;
			float smallOffset = containerSize * 0.017f;

			float gridBubbleSize = containerSize * 0.291f;
			float gridFontSize = containerSize * 0.15f;
			float gridBubbleX = containerSize * 0.125f;
			float gridBubbleY = containerSize * 0.52f;
			CGGlyph gridCodeWidth = (ushort)(containerSize * 0.24f);
			CGGlyph gridCodeHeight = (ushort)(containerSize * 0.5f);

			float gaugeBubbleSize = containerSize * 0.19f;
			float gaugeFontSize = containerSize * 0.10f;
			float gaugeBubbleX = containerSize * 0.5f;
			float gaugeBubbleY = containerSize * 0.17f;
			CGGlyph gaugeCodeWidth = (ushort)(containerSize * 0.105f);
			CGGlyph gaugeCodeHeight = (ushort)(containerSize * 0.275f);

			float labelBubbleSize = containerSize * 0.17f;
			float labelFontSize = containerSize * 0.08f;
			float labelBubbleX = containerSize * 0.57f;
			float labelBubbleY = containerSize * 0.58f;
			CGGlyph labelCodeWidth = (ushort)(containerSize * 0.105f);
			CGGlyph labelCodeHeight = (ushort)(containerSize * 0.275f);

			float greenLineWidth = containerSize * 0.44f;
			float greenLineHeight = containerSize * 0.21f;

			_buttonSize = containerSize * 0.07f;

			float iosLabelFont = containerSize * 0.03f;

			_logoIV = new UIImageView (UIImage.FromBundle ("Images/home_logo.png"));
			_logoIV.ContentMode = UIViewContentMode.ScaleAspectFit;
			this.View.Add (_logoIV);

			_iosLabel = new UILabel ();
			_iosLabel.Text = "INFRAGISTICS iOS CONTROLS";
			_iosLabel.BackgroundColor = UIColor.Clear;
			_iosLabel.TextColor = UIColor.FromWhiteAlpha (145.0f / 255.0f, 1.0f);
			_iosLabel.Font = UIFont.FromName ("HelveticaNeue", iosLabelFont);
			_iosLabel.SizeToFit ();
			this.View.Add (_iosLabel);

			_containerView = new UIView (new RectangleF (this.View.Frame.Size.Width / 2 - containerSize / 2, this.View.Frame.Size.Height / 2 - containerSize / 2, containerSize, containerSize));

			_containerView.BackgroundColor = UIColor.Clear;
			_containerView.Alpha = 0;
			_containerView.AutoresizingMask = UIViewAutoresizing.FlexibleLeftMargin | UIViewAutoresizing.FlexibleTopMargin | UIViewAutoresizing.FlexibleRightMargin| UIViewAutoresizing.FlexibleBottomMargin;
			this.View.Add (_containerView);

			// CHART
			NSArray chartNumbers = NSArray.FromObjects (new[] {
				new NSString ("41"),
				new NSString ("13"),
				new NSString ("10"),
				new NSString ("42")
			});
			NSArray chartFeatures = NSArray.FromObjects (new[] {
				new NSString ("Types"),
				new NSString ("Trend Lines"),
				new NSString ("Markers"),
				new NSString ("Samples")
			});
			RectangleF chartFrame = new RectangleF(padding, padding, chartBubbleSize, chartBubbleSize);
			Bubble chartBubble = CreateBubble (chartNumbers, chartFeatures, chartFrame, false, chartBubbleFontSize, new NSString("Chart"));
			UIImageView chartBadge = SetupImage (new NSString ("Images/Circle-Icons-chart.png"), new RectangleF ((chartBubble.Frame.X + chartBubble.Frame.Size.Width) - largeOffset, chartBubble.ControlNameLabel.Frame.Y - (badgeSize + 5), badgeSize, badgeSize));
			SetupImageCode (new NSString ("Images/Home_chart_code.png"), new RectangleF (chartBubble.Frame.X + chartBubble.Frame.Size.Width + smallOffset, chartBadge.Frame.Y - (chartCodeHeight + smallOffset), chartCodeWidth, chartCodeHeight));

			// GRID
			NSArray gridNumbers = NSArray.FromObjects (new[] {
				new NSString ("40"),
				new NSString ("35"),
				new NSString ("3"),
				new NSString ("2")
			});
			NSArray gridFeatures = NSArray.FromObjects (new[] {
				new NSString ("Features"),
				new NSString ("Samples"),
				new NSString ("DataSources"),
				new NSString ("Themes")
			});
			RectangleF gridFrame = new RectangleF(gridBubbleX, gridBubbleY, gridBubbleSize, gridBubbleSize);
			Bubble gridBubble = CreateBubble (gridNumbers, gridFeatures, gridFrame, true, gridFontSize, new NSString("Grid"));
			SetupImage (new NSString("Images/Circle-Icons-grid.png"), new RectangleF (gridBubble.ControlNameLabel.Frame.X - (badgeSize + smallOffset), gridBubble.ControlNameLabel.Frame.Y, badgeSize, badgeSize));
			SetupImageCode (new NSString("Images/Home_grid_code.png"), new RectangleF (gridBubble.Frame.X - (gridCodeWidth + smallOffset), gridBubble.Frame.Y, gridCodeWidth, gridCodeHeight));

			// GAUGE
			NSArray gaugeNumbers = NSArray.FromObjects (new[] {
				new NSString ("18"),
				new NSString ("56"),
				new NSString ("13"),
				new NSString ("2")
			});
			NSArray gaugeFeatures = NSArray.FromObjects (new[] {
				new NSString ("Features"),
				new NSString ("Needles"),
				new NSString ("Samples"),
				new NSString ("Controls")
			});
			RectangleF gaugeFrame = new RectangleF(gaugeBubbleX, gaugeBubbleY, gaugeBubbleSize, gaugeBubbleSize);
			Bubble gaugeBubble = CreateBubble (gaugeNumbers, gaugeFeatures, gaugeFrame, true, gaugeFontSize, new NSString("Gauge"));
			UIImageView gaugeBadge = SetupImage (new NSString ("Images/Circle-Icons-gauge.png"), new RectangleF ((gaugeBubble.Frame.X + gaugeBubble.Frame.Size.Width) - smallOffset, gaugeBubble.Frame.Y - (badgeSize/2), badgeSize, badgeSize));
			SetupImageCode (new NSString ("Images/Home_gauge_code.png"), new RectangleF (gaugeBadge.Frame.X + gaugeBadge.Frame.Size.Width, gaugeBubble.ControlNameLabel.Frame.Y - gaugeCodeHeight, gaugeCodeWidth, gaugeCodeHeight));

			// WIDGETS
			NSArray labelNumbers = NSArray.FromObjects (new[] {
				new NSString ("8"),
				new NSString ("41")
			});
			NSArray labelFeatues = NSArray.FromObjects (new[] {
				new NSString ("Controls"),
				new NSString ("Samples")
			});
			RectangleF labelFrame = new RectangleF(labelBubbleX, labelBubbleY, labelBubbleSize, labelBubbleSize);
			Bubble labelBubble = CreateBubble (labelNumbers, labelFeatues, labelFrame, false, labelFontSize, new NSString("Widgets"));
			UILabel labelLabel = labelBubble.ControlNameLabel;
			SetupImage (new NSString ("Images/Circle-Icons-widget.png"), new RectangleF (((labelLabel.Frame.X + labelLabel.Frame.Size.Width) - badgeSize) + smallOffset, labelLabel.Frame.Y - badgeSize, badgeSize, badgeSize));
			SetupImageCode (new NSString ("Images/Home_labels_code.png"), new RectangleF (labelLabel.Frame.X - (labelCodeWidth + 5), labelLabel.Frame.Y, labelCodeWidth, labelCodeHeight));

			if(IsPhone)
			{
				labelBubble.FeatureLabel.Frame = new RectangleF ((labelLabel.Frame.X + labelLabel.Frame.Size.Width / 2), labelLabel.Frame.Y + labelLabel.Frame.Size.Height, labelBubble.FeatureLabel.Frame.Size.Width, labelBubble.FeatureLabel.Frame.Size.Height);
			}

			UIImageView greenLines = new UIImageView (UIImage.FromBundle("Images/Home_Graphic_green_lines.png"));
			greenLines.Frame = new RectangleF(chartBubble.ControlNameLabel.Frame.X - greenLineWidth, chartBubble.ControlNameLabel.Frame.Y + smallOffset, greenLineWidth, greenLineHeight);
			_containerView.Add (greenLines);

			_bubbles = NSArray.FromObjects (new[] {
				chartBubble,
				gridBubble,
				gaugeBubble,
				labelBubble
			});

			if(IsPhone)
			{
				_circleSizes = NSArray.FromObjects (new[] {
					NSNumber.FromFloat(40.0f),
					NSNumber.FromFloat(72.0f),
					NSNumber.FromFloat(107.0f),
					NSNumber.FromFloat(165.0f),
					NSNumber.FromFloat(255.0f)
				});
			}
			else
			{
				_circleSizes = NSArray.FromObjects (new[] {
					NSNumber.FromFloat(80.0f),
					NSNumber.FromFloat(145.0f),
					NSNumber.FromFloat(215.0f),
					NSNumber.FromFloat(330.0f),
					NSNumber.FromFloat(510.0f)
				});
			}

			_circle1 = SetupCircleImage (0);
			_circle2 = SetupCircleImage (1);
			_circle3 = SetupCircleImage (2);
			_circle4 = SetupCircleImage (3);

			_circle3_2 = SetupCircleImage (2);
			_circle3_2.Hidden = true;

			_circle4_2 = SetupCircleImage (3);
			_circle4_2.Hidden = true;

			_circle5 = SetupCircleImage (4);

			_circleOverLay = SetupCircleImage (4);
			_circleOverLay.Image = UIImage.FromBundle ("Images/circleLINEoverlay.png");

			_labelUnderline = new UIImageView (UIImage.FromBundle("Images/splash_title_line.png"));
			_labelUnderline.Hidden = IsPhone;
			this.View.Add (_labelUnderline);

			_launchLabelNames = NSArray.FromObjects (new[] {
				new NSString ("Charts"),
				new NSString ("Grids"),
				new NSString("Gauges"),
				new NSString("Widgets")
			});

			_launchLabel = new UILabel ();
			_launchLabel.BackgroundColor = UIColor.Clear;
			_launchLabel.TextColor = UIColor.FromWhiteAlpha (145.0f / 255.0f, 1.0f);
			_launchLabel.Font = UIFont.FromName ("HelveticaNeue", 45.0f);
			this.View.Add (_launchLabel);

			_igLogo = new UIImageView (UIImage.FromBundle("Images/IGlogo.png"));
			_igLogo.ContentMode = UIViewContentMode.ScaleAspectFit;
			_igLogo.Alpha = 0.0f;

			float logoSize = IsPhone ? 75.0f : 110.0f;
			_igLogo.Frame = new RectangleF(this.View.Frame.Size.Width - logoSize, 0, logoSize, logoSize);
			_igLogo.AutoresizingMask = UIViewAutoresizing.FlexibleLeftMargin;
			this.View.Add (_igLogo);
		}

		public override void ViewWillAppear (bool animated)
		{
			base.ViewWillAppear (animated);

			if(_containerView.Alpha == 0)
			{
				_circle2.Transform = _circle4.Transform = CGAffineTransform.MakeRotation (GetRadiansForAngle (360));
				_labelUnderline.Alpha = 0;
				_launchLabel.Alpha = 0;
				_launchLabel.Text = new NSString (_launchLabelNames.ValueAt (0));

				SetupiOSFrame (_buttonSize);

				float duration = 0.5f;

				UIView.Animate (duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
					_launchLabel.Alpha = _labelUnderline.Alpha = 1.0f;
				}, () => {
					UIView.Animate(duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
						_circle5.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(90));
						_circle3.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(90));
					}, () => {
						_launchLabel.Text = new NSString(_launchLabelNames.ValueAt(1));

						UIView.Animate(duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
							_circle4.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(180));
							_circle2.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(180));
						}, () => {
							_launchLabel.Text = new NSString(_launchLabelNames.ValueAt(2));

							UIView.Animate(duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
								_circle5.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(270));
								_circle3.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(270));
							}, () => {
								_launchLabel.Text = new NSString(_launchLabelNames.ValueAt(3));

								UIView.Animate(duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
									_circle2.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(0));
									_circle4.Transform = CGAffineTransform.MakeRotation(GetRadiansForAngle(0));
								}, () => {
									_circle3.Transform = CGAffineTransform.MakeRotation(0);
									_circle4.Transform = CGAffineTransform.MakeRotation(0);

									_circle3_2.Hidden = false;
									_circle4_2.Hidden = false;

									UIView.Animate(duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
										_launchLabel.Alpha = _labelUnderline.Alpha = _circleOverLay.Alpha = _circle1.Alpha = _circle2.Alpha = _circle5.Alpha = 0.0f;

										Bubble chart = (Bubble)NSArray.FromArray<Bubble>(_bubbles).GetValue(0);
										Bubble grid = (Bubble)NSArray.FromArray<Bubble>(_bubbles).GetValue(1);
										Bubble gauge = (Bubble)NSArray.FromArray<Bubble>(_bubbles).GetValue(2);
										Bubble label = (Bubble)NSArray.FromArray<Bubble>(_bubbles).GetValue(3);

										_circle3.Frame = new RectangleF(_containerView.Frame.X + chart.Frame.X, _containerView.Frame.Y + chart.Frame.Y, chart.Frame.Size.Width, chart.Frame.Size.Height);
										_circle4.Frame = new RectangleF(_containerView.Frame.X + grid.Frame.X, _containerView.Frame.Y + grid.Frame.Y, grid.Frame.Size.Width, grid.Frame.Size.Height);

										_circle3_2.Frame = new RectangleF(_containerView.Frame.X + label.Frame.X, _containerView.Frame.Y + label.Frame.Y, label.Frame.Size.Width, label.Frame.Size.Height);
										_circle4_2.Frame = new RectangleF(_containerView.Frame.X + gauge.Frame.X, _containerView.Frame.Y + gauge.Frame.Y, gauge.Frame.Size.Width, gauge.Frame.Size.Height);

										SetupiOSFrame(0);
									}, () => {
										UIView.Animate(duration, 0, UIViewAnimationOptions.CurveEaseInOut, () => {
											_circle4_2.Alpha = _circle3_2.Alpha = _circle3.Alpha = _circle4.Alpha = 0.0f;
											_igLogo.Alpha = _containerView.Alpha = 1.0f;
										}, () => {});
									});
								});
							});
						});
					});
				});
			}
			else
			{
				foreach (Bubble bubble in NSArray.FromArray<Bubble>(_bubbles))
					bubble.Resume ();
			}
		}

		public override void ViewWillDisappear (bool animated)
		{
			base.ViewWillDisappear (animated);

			foreach(Bubble bubble in NSArray.FromArray<Bubble>(_bubbles))
				bubble.Pause();
		}
	}
}

